---
title: 右侧组件
sTitle: dooringx-lib插件开发
order: 13
---

## 右侧组件导入

导入时,只要将开发的组件配成一个对象放入initFormComponents即可。

```js
initFormComponents: Formmodules,
```

## 右侧组件开发

首先为了良好的开发体验，需要定义个formMap类型：

```js
export interface FormBaseType {
	receive?: string;
}
export interface FormInputType extends FormBaseType {
	label: string;
}
export interface FormActionButtonType {}
export interface FormAnimateControlType {}
export interface FormMap {
	input: FormInputType;
	actionButton: FormActionButtonType;
	animateControl: FormAnimateControlType;
}
```
formMap的键名就是initFormComponents键名，formMap的值对应组件需要收到的值。

以input组件为例，FormInputType此时有2个属性，label,receive。

那么在其开发该组件时，props会收到：

```js
interface MInputProps {
	data: CreateOptionsRes<FormMap, 'input'>;
	current: IBlockType;
    config: UserConfig;
}
```
也就是其中data是formMap类型，而current是当前点击的组件，config就不用说了。

还记得在左侧组件开发中的第三个参数吗？这样就都关联起来了：

```js

style: [
			createPannelOptions<FormMap, 'input'>('input', {
				receive: 'text',  
				label: '文字',
			}),
		],

```
createPannelOptions 这个函数的泛型里填入对应的组件，将会给收到的配置项良好的提示。

在配置项组件里所要做的就是接收组件传来的配置项，然后去修改current的属性：


```js
function MInput(props: MInputProps) {
	const option = useMemo(() => {
		return props.data?.option || {};
	}, [props.data]);
	return (
		<Row style={{ padding: '10px 20px' }}>
			<Col span={6} style={{ lineHeight: '30px' }}>
				{(option as any)?.label || '文字'}：
			</Col>
			<Col span={18}>
				<Input
					value={props.current.props[(option as any).receive] || ''}
					onChange={(e) => {
						const receive = (option as any).receive;
						const clonedata = deepCopy(store.getData());
						const newblock = clonedata.block.map((v: IBlockType) => {
							if (v.id === props.current.id) {
								v.props[receive] = e.target.value;
							}
							return v;
						});
						store.setData({ ...clonedata, block: [...newblock] });
					}}
				></Input>
			</Col>
		</Row>
	);
}
```

由于可以很轻松的拿到store，所以可以在任意地方进行修改数据。

将组件的value关联current的属性，onChange去修改store，这样就完成了个双向绑定。

注意：如果你的右侧组件需要用到block以外的属性，可能需要去判断是否处于弹窗模式。